#!/bin/ksh 
# 2010 by Alexander von Gluck at NetIQ UNIX Technical Support
#
# Revision 4 Aug/2010
# * added export LIBPATH . for older AIX installs
# * limited shared memory segments only show SPMI (KEY 0x78)
# * now shows applications accessing SPMI (as per IBM technical support)
# * removed -r from oslevel to prevent lockups
#
# Revision 3 Jul/2010
# * changed to run with ksh instead of bash.. bash is awesome but IBM doesn't agree.
# * implimented getSPMI function to save space
#
# Revision 2 Nov/2009
# * Initial public release
#
# This is an unofficial script produced by Technical Support
# Support for this script will be limited.

function getSPMI {
	result=`./retrieveAIXCtr -s "$1" "$2" "$3";`

	if [[ $? -ne 0 || $result -lt 0 ]]  ; then
		echo "    $1 - $2 - instance $3 : ERROR";
	else
		echo "    $1 - $2 - instance $3 : $result";
	fi
}

echo "-------------------------------------------"
echo "AIX Performance Counter Testing Application"
echo "2009 NetIQ UNIX Technical Support"
echo "Revision 4 (Aug/2010)"
echo
echo "Run at:"
date
echo

if [ $(id -u) != 0 ]; then
	echo "ERROR: Please run this counter testing script as root";
fi

# set library path of local directory for needed NetIQ libs
export LD_LIBRARY_PATH=$LD_LIBRARY_PATH:.
export LIBPATH=$LIBPATH:.

echo "-------------------"
echo "System is:"
oslevel
echo
echo "-------------------"
echo "Performance libraries installed:"
lslpp -h -a -c | grep perfstat

# 0x78 == SPMI
echo
echo "-------------------"
echo "SPMI shared memory segments:"
echo "T        ID     KEY        MODE       OWNER    GROUP  CREATOR   CGROUP NATTCH     SEGSZ  CPID  LPID   ATIME    DTIME    CTIME"
ipcs -a | grep '0x78'

echo
echo "-------------------"
echo "Applications currently using SPMI:"
genld -l | grep -p Spmi | grep Proc

if [ $? -ne 0 ] ; then 
	echo "   ERROR: there is a serious shared memory problem on this system, ipcs is not functioning."
fi

#
# Common counter tests...
#
echo
echo "-------------------"
echo "Begining SPMI counter tests..."

IFS="*"

# this really should be an array which we for each but AIX's sh has been fighting me today

cpucount=`lsdev -C -c processor | wc -l`;

cpustorun=`echo "$cpucount - 1" | bc`

echo "\n CPU related counters"
getSPMI "UX Processor" "%Utilization" "_Total";

# for's in bash are way better, i'm so spoiled
integer i=0
while ((i <= $cpustorun));
do
	getSPMI "UX Processor" "%Utilization" "$i";
	getSPMI "UX Processor" "%System Time" "$i";
	getSPMI "UX Processor" "%User Time" "$i";
	getSPMI "UX Processor" "%Idle Time" "$i";
	getSPMI "UX Processor" "%Wait Time" "$i";
	(( i = i + 1));
done

getSPMI "UX Processor" "Context Switch/s" "_Total";
getSPMI "UX Processor" "System Call/s" "_Total";
getSPMI "UX Processor" "Run Queue Length/s" "_Total";

echo "\n Paging space related counters (used by UNIX_PagingHigh)"
getSPMI "UX Paging" "Page-in KBytes/s" "_Total";
getSPMI "UX Paging" "Page-out KBytes/s" "_Total";

echo "\n Disk related counters (used by UNIX_PhysicalDiskIO)"
getSPMI "UX Disk" "Kilobytes Read/s" "_Total";
getSPMI "UX Disk" "Kilobytes Write/s" "_Total";

disks=`lspv | awk '{ print $1 }'`
for disk in $disks;
do
	getSPMI "UX Disk" "Kilobytes Read/s" "$disk";
	getSPMI "UX Disk" "Kilobytes Write/s" "$disk";
done

echo "\n Network related counters"
getSPMI "UX Networking" "TCP Connections" "_Total";
getSPMI "UX Networking" "Established Connections" "_Total";

echo
echo "-------------------"
echo "Counter testing script complete."
echo "Please return the output above to NetIQ technical support."
echo 
echo "If any values above are non-positive numbers or say ERROR, there is a problem with that counter";
echo "-------------------"

# done
